@@ -219,10 +219,12 @@ module VagrantPlugins
               networks_to_configure << network
             end
 
-            env[:ui].info I18n.t('vagrant.actions.vm.network.configuring')
-            env[:machine].guest.capability(
-              :configure_networks, networks_to_configure
-            )
+            unless networks_to_configure.empty?
+              env[:ui].info I18n.t('vagrant.actions.vm.network.configuring')
+              env[:machine].guest.capability(
+                :configure_networks, networks_to_configure
+              )
+            end
 
           end
         end
@@ -281,7 +283,7 @@ module VagrantPlugins
             return options[:network_name]
           end
 
-          # Get list of all (active and inactive) libvirt networks.
+          # Get list of all (active and inactive) Libvirt networks.
           available_networks = libvirt_networks(libvirt_client)
 
           return 'public' if options[:iface_type] == :public_network
