@@ -23,7 +23,7 @@ module VagrantPlugins
 
             libvirt_domain = env[:machine].provider.driver.connection.client.lookup_domain_by_uuid(env[:machine].id)
 
-            # libvirt API doesn't support modifying memory on NUMA enabled CPUs
+            # Libvirt API doesn't support modifying memory on NUMA enabled CPUs
             # http://libvirt.org/git/?p=libvirt.git;a=commit;h=d174394105cf00ed266bf729ddf461c21637c736
             if config.numa_nodes == nil
               if config.memory.to_i * 1024 != libvirt_domain.max_memory
@@ -68,7 +68,7 @@ module VagrantPlugins
               end
 
               # vCpu count
-              if config.cpus.to_i != libvirt_domain.vcpus.length
+              if config.cpus.to_i != libvirt_domain.num_vcpus(0)
                 descr_changed = true
                 REXML::XPath.first(xml_descr, '/domain/vcpu').text = config.cpus
               end
