@@ -11,7 +11,6 @@ describe 'templates/domain' do
 
     def finalize!
       super
-      @qargs = @qemu_args
     end
   end
 
@@ -64,7 +63,7 @@ describe 'templates/domain' do
                      source_path: '/tmp/foo')
       domain.random(model: 'random')
       domain.pci(bus: '0x06', slot: '0x12', function: '0x5')
-      domain.pci(bus: '0x03', slot: '0x00', function: '0x0')
+      domain.pci(domain: '0x0001', bus: '0x03', slot: '0x00', function: '0x0')
       domain.usb_controller(model: 'nec-xhci', ports: '4')
       domain.usb(bus: '1', device: '2', vendor: '0x1234', product: '0xabcd')
       domain.redirdev(type: 'tcp', host: 'localhost', port: '4000')
@@ -76,6 +75,15 @@ describe 'templates/domain' do
 
       domain.qemuargs(value: '-device')
       domain.qemuargs(value: 'dummy-device')
+
+      domain.qemuenv(QEMU_AUDIO_DRV: 'pa')
+      domain.qemuenv(QEMU_AUDIO_TIMER_PERIOD: '150')
+      domain.qemuenv(QEMU_PA_SAMPLES: '1024')
+      domain.qemuenv(QEMU_PA_SERVER: '/run/user/1000/pulse/native')
+
+      domain.shares = '1024'
+      domain.cpuset = '1-4,^3,6'
+      domain.nodeset = '1-4,^3,6'
     end
     let(:test_file) { 'domain_all_settings.xml' }
     it 'renders template' do
